from __future__ import annotations

import collections.abc
import types
import typing

import PySide6.QtGui

__all__: list[str] = ["Matrix4x4"]

class Matrix4x4:
    """
    A 4x4 transformation matrix.
    """

    __hash__: typing.ClassVar[None] = None  # type: ignore
    @staticmethod
    def identity_matrix() -> Matrix4x4:
        """
        Construct a new identity matrix.
        """

    @staticmethod
    def rotation_x_matrix(rx: typing.SupportsFloat) -> Matrix4x4:
        """
        Construct a new rotation matrix in the x axis.
        """

    @staticmethod
    def rotation_y_matrix(ry: typing.SupportsFloat) -> Matrix4x4:
        """
        Construct a new rotation matrix in the y axis.
        """

    @staticmethod
    def rotation_z_matrix(rz: typing.SupportsFloat) -> Matrix4x4:
        """
        Construct a new rotation matrix in the z axis.
        """

    @staticmethod
    def scale_matrix(
        sx: typing.SupportsFloat, sy: typing.SupportsFloat, sz: typing.SupportsFloat
    ) -> Matrix4x4:
        """
        Construct a new scale matrix.
        """

    @staticmethod
    def transformation_matrix(
        sx: typing.SupportsFloat,
        sy: typing.SupportsFloat,
        sz: typing.SupportsFloat,
        rx: typing.SupportsFloat,
        ry: typing.SupportsFloat,
        rz: typing.SupportsFloat,
        dx: typing.SupportsFloat,
        dy: typing.SupportsFloat,
        dz: typing.SupportsFloat,
    ) -> Matrix4x4:
        """
        Construct a new transformation matrix made from scale, rotation and translation.
        """

    @staticmethod
    def translation_matrix(
        dx: typing.SupportsFloat, dy: typing.SupportsFloat, dz: typing.SupportsFloat
    ) -> Matrix4x4:
        """
        Construct a new translation matrix.
        """

    @typing.overload
    def __eq__(self, other: Matrix4x4) -> bool: ...
    @typing.overload
    def __eq__(self, other: typing.Any) -> bool | types.NotImplementedType: ...
    @typing.overload
    def __init__(self) -> None:
        """
        Construct an identity matrix.
        """

    @typing.overload
    def __init__(
        self,
        arg0: tuple[
            tuple[
                typing.SupportsFloat,
                typing.SupportsFloat,
                typing.SupportsFloat,
                typing.SupportsFloat,
            ],
            tuple[
                typing.SupportsFloat,
                typing.SupportsFloat,
                typing.SupportsFloat,
                typing.SupportsFloat,
            ],
            tuple[
                typing.SupportsFloat,
                typing.SupportsFloat,
                typing.SupportsFloat,
                typing.SupportsFloat,
            ],
            tuple[
                typing.SupportsFloat,
                typing.SupportsFloat,
                typing.SupportsFloat,
                typing.SupportsFloat,
            ],
        ],
    ) -> None:
        """
        Construct from tuples.
        """

    @typing.overload
    def __init__(self, other: Matrix4x4) -> None:
        """
        Copy from another matrix.
        """

    @typing.overload
    def __init__(self, arg0: collections.abc.Buffer) -> None: ...
    @typing.overload
    def __init__(self, arg0: PySide6.QtGui.QMatrix4x4) -> None: ...
    @typing.overload
    def __mul__(self, other: Matrix4x4) -> Matrix4x4:
        """
        Multiply this matrix with another matrix.
        """

    @typing.overload
    def __mul__(
        self,
        other: list[
            tuple[typing.SupportsFloat, typing.SupportsFloat, typing.SupportsFloat]
        ],
    ) -> list[tuple[float, float, float]]:
        """
        Multiply this matrix with a sequence of vectors.
        """

    def __repr__(self) -> str: ...
    def almost_equal(self, other: Matrix4x4, err: typing.SupportsFloat = 1e-06) -> bool:
        """
        Check if this matrix is almost equal to another matrix.
        """

    def decompose(
        self,
    ) -> tuple[
        tuple[float, float, float],
        tuple[float, float, float],
        tuple[float, float, float],
    ]:
        """
        Decompose the matrix into scale, rotation and displacement tuples.
        Note that these values may be incorrect if the matrix is more complex
        Recompose the matrix and compare with the original to check
        """

    def get_element(self, i: typing.SupportsInt, j: typing.SupportsInt) -> float:
        """
        Get an element in the matrix.
        """

    def inverse(self) -> Matrix4x4:
        """
        Compute the inverse of this matrix.
        Raises RuntimeError if the matrix cannot be inverted.
        """

    def rotate_x(self, rx: typing.SupportsFloat) -> Matrix4x4:
        """
        Rotate this matrix by the specified amount in the x axis.
        """

    def rotate_y(self, ry: typing.SupportsFloat) -> Matrix4x4:
        """
        Rotate this matrix by the specified amount in the y axis.
        """

    def rotate_z(self, rz: typing.SupportsFloat) -> Matrix4x4:
        """
        Rotate this matrix by the specified amount in the z axis.
        """

    def scale(
        self,
        sx: typing.SupportsFloat,
        sy: typing.SupportsFloat,
        sz: typing.SupportsFloat,
    ) -> Matrix4x4:
        """
        Scale this matrix by the specified amount.
        """

    def set_element(
        self, i: typing.SupportsInt, j: typing.SupportsInt, value: typing.SupportsFloat
    ) -> None:
        """
        Set an element in the matrix.
        """

    def to_qt(self) -> PySide6.QtGui.QMatrix4x4: ...
    def translate(
        self,
        dx: typing.SupportsFloat,
        dy: typing.SupportsFloat,
        dz: typing.SupportsFloat,
    ) -> Matrix4x4:
        """
        Translate this matrix by the specified amount.
        """



from __future__ import annotations

import collections.abc

import amulet.utils.event

__all__: list[str] = [
    "AbstractCancelManager",
    "CancelManager",
    "TaskCancelled",
    "VoidCancelManager",
]

class AbstractCancelManager:
    def cancel(self) -> None:
        """
        Request the operation be cancelled.
        It is down to the operation to implement support for this.
        Thread safe.
        """

    def is_cancel_requested(self) -> bool:
        """
        Has :meth:`cancel` been called to signal that the operation should be cancelled.
        Thread safe.
        """

    def register_cancel_callback(
        self, callback: collections.abc.Callable[[], None]
    ) -> amulet.utils.event.EventToken[()]:
        """
        Register a function to get called when cancel is called.
        The callback will be called from the thread `cancel` is called in.
        Thread safe.
        """

    def unregister_cancel_callback(
        self, token: amulet.utils.event.EventToken[()]
    ) -> None:
        """
        Unregister a registered function from being called when cancel is called.
        Thread safe.
        """

class CancelManager(AbstractCancelManager):
    def __init__(self) -> None: ...
    def __repr__(self) -> str: ...

class TaskCancelled(Exception):
    pass

class VoidCancelManager(AbstractCancelManager):
    def __init__(self) -> None: ...
    def __repr__(self) -> str: ...



from __future__ import annotations

from amulet.utils.task_manager.cancel_manager import (
    AbstractCancelManager,
    CancelManager,
    TaskCancelled,
    VoidCancelManager,
)
from amulet.utils.task_manager.progress_manager import (
    AbstractProgressManager,
    ProgressManager,
    VoidProgressManager,
)

from . import cancel_manager, progress_manager

__all__: list[str] = [
    "AbstractCancelManager",
    "AbstractProgressManager",
    "CancelManager",
    "ProgressManager",
    "TaskCancelled",
    "VoidCancelManager",
    "VoidProgressManager",
    "cancel_manager",
    "progress_manager",
]



from __future__ import annotations

from . import (
    _amulet_utils,
    _version,
    event,
    image,
    lock,
    logging,
    matrix,
    numpy,
    task_manager,
)

__all__: list[str] = [
    "compiler_config",
    "event",
    "image",
    "lock",
    "logging",
    "matrix",
    "numpy",
    "task_manager",
]

def _init() -> None: ...

__version__: str
compiler_config: dict



from __future__ import annotations

import collections.abc

import numpy
import numpy.typing

__all__: list[str] = ["unique_inverse"]

def unique_inverse(
    array: collections.abc.Buffer,
) -> tuple[numpy.typing.NDArray[numpy.uint32], numpy.typing.NDArray[numpy.uint32]]: ...



from __future__ import annotations

import typing

__all__: list[str] = ["ConnectionMode"]

class ConnectionMode:
    """
    Members:

      Direct : Directly called by the emitter.

      Async : Called asynchronously.
    """

    Async: typing.ClassVar[
        ConnectionMode
    ]  # value = amulet.utils.event._connection_mode.ConnectionMode.Async
    Direct: typing.ClassVar[
        ConnectionMode
    ]  # value = amulet.utils.event._connection_mode.ConnectionMode.Direct
    __members__: typing.ClassVar[
        dict[str, ConnectionMode]
    ]  # value = {'Direct': amulet.utils.event._connection_mode.ConnectionMode.Direct, 'Async': amulet.utils.event._connection_mode.ConnectionMode.Async}
    def __eq__(self, other: typing.Any) -> bool: ...
    def __hash__(self) -> int: ...
    def __index__(self) -> int: ...
    def __init__(self, value: typing.SupportsInt) -> None: ...
    def __int__(self) -> int: ...
    def __ne__(self, other: typing.Any) -> bool: ...
    def __repr__(self) -> str: ...
    def __str__(self) -> str: ...
    @property
    def name(self) -> str: ...
    @property
    def value(self) -> int: ...



from __future__ import annotations

import typing

import amulet.utils.event

from . import _logging

__all__: list[str] = [
    "get_logger",
    "get_min_log_level",
    "register_default_log_handler",
    "set_min_log_level",
    "unregister_default_log_handler",
]

def get_logger() -> amulet.utils.event.Event[int, str]:
    """
    Get the logger event.
    This is emitted with the message and its level every time a message is logged.
    """

def get_min_log_level() -> int:
    """
    Get the maximum message level that will be logged.
    Registered handlers may be more strict.
    Thread safe.
    """

def register_default_log_handler() -> None:
    """
    Register the default log handler.
    This is registered by default with a log level of 20.
    Thread safe.
    """

def set_min_log_level(level: typing.SupportsInt) -> None:
    """
    Set the maximum message level that will be logged.
    Registered handlers may be more strict.
    Thread safe.
    """

def unregister_default_log_handler() -> None:
    """
    Unregister the default log handler.
    Thread safe.
    """



import logging as _logging

from . import _version

__version__ = _version.get_versions()["version"]

# init a default logger
_logging.basicConfig(level=_logging.INFO, format="%(levelname)s - %(message)s")


def _init() -> None:
    import os
    import sys
    import ctypes
    import platformdirs

    os.environ.setdefault(
        "CACHE_DIR", platformdirs.user_cache_dir("AmuletTeam", "AmuletTeam")
    )
    os.makedirs(os.environ["CACHE_DIR"], exist_ok=True)

    if sys.platform == "win32":
        lib_path = os.path.join(os.path.dirname(__file__), "amulet_utils.dll")
    elif sys.platform == "darwin":
        lib_path = os.path.join(os.path.dirname(__file__), "libamulet_utils.dylib")
    elif sys.platform == "linux":
        lib_path = os.path.join(os.path.dirname(__file__), "libamulet_utils.so")
    else:
        raise RuntimeError(f"Unsupported platform {sys.platform}")

    # Load the shared library
    ctypes.cdll.LoadLibrary(lib_path)

    from ._amulet_utils import init

    init(sys.modules[__name__])


_init()



from __future__ import annotations

import PIL.Image

__all__: list[str] = ["get_missing_no_icon", "missing_no_icon_path"]

def get_missing_no_icon() -> PIL.Image.Image: ...

missing_no_icon_path: str



from __future__ import annotations

import typing as _typing
from . import _connection_mode
from ._connection_mode import ConnectionMode

if _typing.TYPE_CHECKING:
    import collections.abc


_Args = _typing.TypeVarTuple("_Args")


class EventToken(_typing.Protocol[*_Args]):
    pass


@_typing.runtime_checkable
class Event(_typing.Protocol[*_Args]):
    def connect(
        self,
        callback: collections.abc.Callable[[*_Args], None],
        mode: ConnectionMode = ConnectionMode.Direct,
    ) -> EventToken[*_Args]:
        """
        Connect a callback to this event and return a token.
        The token must be kept alive for the callback to work.
        The token is used to disconnect the callback when it is not needed.
        Thread safe.
        """

    def disconnect(self, token: EventToken[*_Args]) -> None:
        """
        Disconnect a callback.
        Token is the value returned by connect.
        Thread safe.
        """

    def dispatch(self, *args: *_Args) -> None:
        """
        Call all callbacks with the given arguments from this thread.
        Blocks until all callbacks are processed.
        Thread safe.
        """
